@@ -28,13 +28,12 @@ _gtkbase.gtk_set_locale()
 # initialise GTK (only if it can find the display -- this avoids segfaults)
 if '--display' in sys.argv or os.environ.has_key('DISPLAY'):
 	_gtkbase.gtk_init()
+	# this makes the _gtk module aware of the types and funcs in _gtkbase
+	_gtk.pygtk_init()
 else:
 	print 'No display information -- gtk_init not called'
 del sys, os
 
-# this makes the _gtk module aware of the types and functions in _gtkbase
-_gtk.pygtk_init()
-
 class GtkObject:
 	get_type = _gtkbase.gtk_object_get_type
 	def __init__(self, _obj=None):
@@ -162,8 +161,42 @@ class GtkTooltips(GtkData):
 	def set_colors(self, back, fore):
 		_gtk.gtk_tooltips_set_colors(self._o, back, fore)
 
+class GtkItemFactory(GtkObject):
+	get_type = _gtk.gtk_item_factory_get_type
+	def __init__(self, type=_gtk.gtk_menu_bar_get_type(), path='',
+		     accel_group=None, _obj=None):
+		if _obj: self._o = _obj; return
+		if hasattr(type, 'get_type'): type = type.get_type()
+		self._o = _gtk.gtk_item_factory_new(type,path,accel_group._ag)
+	parse_rc = _gtk.gtk_item_factory_parse_rc
+	parse_rc_string = _gtk.gtk_item_factory_parse_rc_string
+	def get_widget(self, path):
+		return _obj2inst(_gtk.gtk_item_factory_get_widget(self._o,
+								  path))
+	def get_widget_by_action(self, action):
+		return _obj2inst(_gtk.gtk_item_factory_get_widget_by_action(
+			self._o, action))
+	class __wrap_cb:
+		def __init__(self, cb):
+			self.cb = cb
+		def __call__(self, action, widget):
+			self.cb(action, _obj2inst(widget))
+	def create_items(self, items):
+		def map_func((path,accel,cb,action,type), wrap=self.__wrap_cb):
+			if cb:
+				return (path,accel,wrap(cb).__call__,action,
+					type)
+			else:
+				return (path,accel,None,action,type)
+		items = map(map_func, items)
+		_gtk.gtk_item_factory_create_items(self._o, items)
+	def delete_item(self, path):
+		_gtk.gtk_item_factory_delete_item(self._o, path)
+	def popup(self, x, y, button, time):
+		_gtk.gtk_item_factory_popup(self._o, x, y, button, time)
+
 class GtkWidget(GtkObject):
-	_gtkbase.gtk_widget_get_type
+	get_type = _gtkbase.gtk_widget_get_type
 	def __init__(self, _obj=None):
 		if _obj: self._o = _obj; return
 	# extra parameter is so that these funcs can be used as signal funcs
@@ -301,7 +334,7 @@ class GtkWidget(GtkObject):
 		_gtkbase.gtk_widget_unrealize(self._o)
 	
 class GtkContainer(GtkWidget):
-	_gtkbase.gtk_container_get_type
+	get_type = _gtkbase.gtk_container_get_type
 	def __init__(self, _obj=None):
 		if _obj: self._o = _obj; return
 	def add(self, child, **args):
@@ -1225,7 +1258,7 @@ class GtkVPaned(GtkPaned):
 		if _obj: self._o = _obj; return
 		self._o = _gtk.gtk_vpaned_new()
 
-class GtkScrolledWindow(GtkContainer):
+class GtkScrolledWindow(GtkViewport):
 	get_type = _gtk.gtk_scrolled_window_get_type
 	def __init__(self, hadj=None, vadj=None, _obj=None):
 		if _obj: self._o = _obj; return
@@ -1472,7 +1505,7 @@ class GtkSpinButton(GtkEntry):
 	def __init__(self, adj=None, climb_rate=1, digits=1, _obj=None):
 		if _obj: self._o = _obj; return
 		if adj:
-			self._o = _gtk.gtk_spin_button_new(adj, climb_rate,
+			self._o = _gtk.gtk_spin_button_new(adj._o, climb_rate,
 							   digits)
 		else:
 			self._o = _gtk.gtk_spin_button_new_no_adj(climb_rate,
@@ -1626,11 +1659,9 @@ class GtkTipsQuery(GtkLabel):
 
 class GtkPreview(GtkWidget):
 	get_type = _gtk.gtk_preview_get_type
-	def uninit(self):
-		_gtk.gtk_preview_uninit()
-	def __init__(self, tp=0):
+	def __init__(self, type=PREVIEW_COLOR, _obj=None):
 		if _obj: self._o = _obj; return
-		self._o = _gtk.gtk_preview_new(tp)
+		self._o = _gtk.gtk_preview_new(type)
 	def size(self, w, h):
 		_gtk.gtk_preview_size(self._o, w, h)
 	def put(self, win, gc, srcx, srcy, dstx, dsty, width, height):
